Package: dtrackr
Title: Track your Data Pipelines
Version: 0.5.0
Authors@R: 
    person(given = "Robert",
           family = "Challen",
           role = c("aut", "cre"),
           email = "rob.challen@bristol.ac.uk",
           comment = c(ORCID = "0000-0002-5504-7768"))
Description: Track and
    document 'dplyr' data pipelines. As you filter, mutate, and join your
    way through a data set, 'dtrackr' seamlessly keeps track of your data
    flow and makes publication ready documentation of a data pipeline simple.
License: MIT + file LICENSE
Language: en-GB
Imports: dplyr (>= 1.1.0), glue, htmltools, magrittr, rlang, rsvg,
        stringr, tibble, tidyr, utils, V8, fs, purrr, base64enc,
        pdftools, png, lifecycle, scales
Suggests: spelling, here, knitr, rmarkdown, tidyselect, devtools,
        testthat (>= 2.1.0), rstudioapi, survival, ggplot2, covr
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.2.9004
Depends: R (>= 2.10)
URL: https://terminological.github.io/dtrackr/index.html,
        https://github.com/terminological/dtrackr
BugReports: https://github.com/terminological/dtrackr/issues
NeedsCompilation: no
Packaged: 2025-09-01 15:08:52 UTC; vp22681
Author: Robert Challen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5504-7768>)
Maintainer: Robert Challen <rob.challen@bristol.ac.uk>
Repository: CRAN
Date/Publication: 2025-09-01 15:50:07 UTC
Built: R 4.6.0; ; 2026-04-26 17:46:49 UTC; unix
