# Generated by roxygen2: do not edit by hand

S3method(add_count,trackr_df)
S3method(anti_join,trackr_df)
S3method(arrange,trackr_df)
S3method(distinct,trackr_df)
S3method(filter,trackr_df)
S3method(full_join,trackr_df)
S3method(group_by,trackr_df)
S3method(group_modify,trackr_df)
S3method(inner_join,trackr_df)
S3method(intersect,trackr_df)
S3method(left_join,trackr_df)
S3method(mutate,trackr_df)
S3method(nest,trackr_df)
S3method(nest_join,trackr_df)
S3method(pivot_longer,trackr_df)
S3method(pivot_wider,trackr_df)
S3method(plot,trackr_graph)
S3method(print,trackr_graph)
S3method(reframe,trackr_df)
S3method(relocate,trackr_df)
S3method(rename,trackr_df)
S3method(rename_with,trackr_df)
S3method(right_join,trackr_df)
S3method(select,trackr_df)
S3method(semi_join,trackr_df)
S3method(setdiff,trackr_df)
S3method(slice,trackr_df)
S3method(slice_head,trackr_df)
S3method(slice_max,trackr_df)
S3method(slice_min,trackr_df)
S3method(slice_sample,trackr_df)
S3method(slice_tail,trackr_df)
S3method(summarise,trackr_df)
S3method(transmute,trackr_df)
S3method(ungroup,trackr_df)
S3method(union,trackr_df)
S3method(union_all,trackr_df)
S3method(unnest,trackr_df)
export("%>%")
export(add_tally)
export(bind_cols)
export(bind_rows)
export(capture_exclusions)
export(comment)
export(count_subgroup)
export(dot2svg)
export(exclude_all)
export(excluded)
export(filter)
export(flowchart)
export(history)
export(include_any)
export(p_add_count)
export(p_add_tally)
export(p_anti_join)
export(p_arrange)
export(p_bind_cols)
export(p_bind_rows)
export(p_capture_exclusions)
export(p_clear)
export(p_comment)
export(p_copy)
export(p_count_if)
export(p_count_subgroup)
export(p_distinct)
export(p_exclude_all)
export(p_excluded)
export(p_filter)
export(p_flowchart)
export(p_full_join)
export(p_get)
export(p_get_as_dot)
export(p_group_by)
export(p_group_modify)
export(p_include_any)
export(p_inner_join)
export(p_intersect)
export(p_left_join)
export(p_mutate)
export(p_nest)
export(p_nest_join)
export(p_pause)
export(p_pivot_longer)
export(p_pivot_wider)
export(p_reframe)
export(p_relocate)
export(p_rename)
export(p_rename_with)
export(p_resume)
export(p_right_join)
export(p_select)
export(p_semi_join)
export(p_set)
export(p_setdiff)
export(p_slice)
export(p_slice_head)
export(p_slice_max)
export(p_slice_min)
export(p_slice_sample)
export(p_slice_tail)
export(p_status)
export(p_summarise)
export(p_tagged)
export(p_track)
export(p_transmute)
export(p_ungroup)
export(p_union)
export(p_union_all)
export(p_unnest)
export(p_untrack)
export(pause)
export(resume)
export(save_dot)
export(status)
export(std_size)
export(tagged)
export(track)
export(untrack)
importFrom(dplyr,add_count)
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,group_modify)
importFrom(dplyr,inner_join)
importFrom(dplyr,intersect)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,nest_join)
importFrom(dplyr,reframe)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,semi_join)
importFrom(dplyr,setdiff)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_max)
importFrom(dplyr,slice_min)
importFrom(dplyr,slice_sample)
importFrom(dplyr,slice_tail)
importFrom(dplyr,summarise)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(dplyr,union)
importFrom(dplyr,union_all)
importFrom(lifecycle,deprecated)
importFrom(magrittr,"%>%")
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unnest)
